/*
 * Copyright (c) 2005, David Peterson 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 * 			 notice, this list of conditions and the following disclaimer in the
 *   		 documentation and/or other materials provided with the distribution.
 *     * Neither the name of "randombits.org" nor the names of its contributors
 * 			 may be used to endorse or promote products derived from this software
 * 			 without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * v1.0: Created on 16/05/2005 by David Peterson
 */
package org.randombits.confluence.metadata.reference;

import com.atlassian.user.User;
import org.apache.commons.lang.StringUtils;

/**
 * @author David Peterson
 * @version 2.0
 */
public class UserReference implements Reference<User> {

    private static final long serialVersionUID = 1934030029425758042L;

    private String username;

    /**
     * Constructs a new, blank user option. For use when defrosting.
     */
    protected UserReference() {
    }

    public UserReference( User user ) {
        this( user.getName() );
    }

    /**
     * Constructs a new user option.
     *
     * @param username The username to provide an option for.
     */
    public UserReference( String username ) {
        this.username = username;
    }

    public String getUsername() {
        return username;
    }

    /**
     * @param obj The object to compare.
     * @return <code>true</code> if the object is a UserOption with the same
     *         username.
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals( Object obj ) {
        if ( obj instanceof UserReference ) {
            UserReference uo = (UserReference) obj;
            return StringUtils.equals( username, uo.username );
        }
        return false;
    }

    /**
     * @return the hashcode for the object.
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        return 32 + ( ( username != null ) ? username.hashCode() : 0 );
    }

    /**
     * @return the String version of the list option.
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "user: " + username;
    }
}
